<?php

namespace Framework\Controllers;

use Framework\Foundation\Config;
use Framework\Modules\UserAccess;
use Framework\Modules\PaayThreeDs;
use Framework\Modules\Crm\Adapter As CrmAdapter;
use GuzzleHttp\Client;

class BaseController
{

    protected $templateEngine;
    protected $templateToken;
    protected $apiType;
    protected $crmAdapter;
    protected $httpClient;

    public function __construct()
    {
        $this->templateEngine = new \Framework\Foundation\TemplateEngine();
        $this->templateToken  = new \Framework\Modules\TemplateToken();

        $this->apiType    = Config::get('payment_provider.api_type');
        $this->crmAdapter = new CrmAdapter(app()->crmInstance());
        $this->httpClient = new Client([
            'strict'          => false,
            'protocols'       => ['http', 'https'],
            'allow_redirects' => true,
            'defaults'        => ['verify' => false],
        ]);
    }

    /**
     * Parse all public configs
     * @return Array
     */
    public function config()
    {
        $config = Config::getAllPublic();

        $upsellStep = 0;
        if (Config::get('offer_details.enable_upsells'))
        {
            $upsellStep = count(Config::get('offer_details')) - 1;
        }

        $enable_upsell_step = 0;
        if ($upsellStep > 0 && Config::get('offer_details.enable_upsells') == 1)
        {
            $offer_details       = Config::get('offer_details');
            $upsell_current_step = (int) $this->getCurrentStep('upsell');
            $enable_upsell_step  = (!empty($offer_details['step' . $upsell_current_step]['splitChargeDetails']) ? 1 : 0);
        }

        $config['enable_upsell_step']  = $enable_upsell_step;
        $config['upsell_current_step'] = (int) $this->getCurrentStep('upsell');
        $config['upsell_step']         = $upsellStep;

        $config['productStore']                   = $this->prepareProducts();
        $config['currency_rates_update_required'] = \Framework\Modules\CurrencyRates::isUpdateRequired();

        // Checking paay 3d
        $config['isActivePaay'] = PaayThreeDs::isActivePaay();

        if (Config::get('payment_provider.api_type') === 'checkout')
        {
            $config['checkout_public_key'] = Config::get('payment_provider.api_user_name');
        }

        $config['show_google_captcha'] = UserAccess::_showGoogleCaptcha();

        $config['paypal_express'] = Config::get('addons.paypal_express');

        return $config;
    }

    /**
     * Get all public template tokens
     * 
     * @return Array
     */
    protected function templateToken($request)
    {
        $token = $this->templateToken->updateToken($request);

        // Add any updated token here
        $coreAppJsLinks = array_get($token, 'core_app_js_links', '');

        // List all depricated tokens
        $token['elm_configs'] = "<script> console.log('%cThe [[elm_configs | raw]] token is depricated and will be removed from v4.1.0', 'color: #ff6a00; font-size: 18px'); </script>";

        $token['core_app_js_links'] = sprintf("<script> window.UniConfig = %s </script>", json_encode($this->config()));
        $token['core_app_js_links'] .= $coreAppJsLinks;

        if (Config::get('api_type') === 'checkout')
        {
            $token['core_app_js_links'] .= '<script src="https://cdn.checkout.com/js/frames.js"></script><script src="frontend/js/checkout_gateway_ui.js"></script>';
        }

        if (Config::get('addons.cookie_consent.is_active'))
        {
            $token['core_app_js_links'] .= $token['cookie_consent'];
        }

        $token['ng_app_id'] = Config::get('ng_app_id', '5edf407dd0e6');

        return $token;
    }

    /**
     * Get funnel current step (index|checkout:1, upsell1:2, upsell2:3, ...)
     * @param string $page
     * @return integer
     */
    protected function getCurrentStep($page = '')
    {
        if ($page == 'index' || $page == 'checkout')
        {
            $currentStep = 1;
        }
        elseif ($page == 'upsell')
        {
            if (!isset($_SESSION['upsell_step']))
            {
                $currentStep = 2;
            }
            else
            {
                $currentStep = $_SESSION['upsell_step'];
            }
        }
        else
        {
            $currentStep = 1;
        }

        return $currentStep;
    }

    /**
     * Send mail
     * @param Array $param
     * 
     * @return Array
     */
    protected function sendMail(Array $param)
    {
        try
        {
            $to      = array_get($param, 'to');
            $subject = array_get($param, 'subject');
            $message = wordwrap(array_get($param, 'message', ''), 70);
            $from    = array_get($param, 'from');

            $headers = 'From: ' . $from . "\r\n" .
                    'Reply-To: ' . $from . "\r\n" .
                    'X-Mailer: PHP/' . phpversion();
            $headers .= 'Content-type: text/html; charset=iso-8859-1' . "\r\n";
            $headers .= 'Content-type: text/html; charset=iso-8859-1' . "\r\n";

            if (empty($to))
                throw new \Exception('Required mail To address');
            if (empty($subject))
                throw new \Exception('Required mail subject');
            if (empty($message))
                throw new \Exception('Required mail body');
            if (empty($from))
                throw new \Exception('Required mail From address');

            return mail($to, $subject, $message, $headers);
        }
        catch (\Exception $e)
        {
            if (APP_DEBUG)
            {
                throw new \Exception($e->getMessage());
            }
            return false;
        }

        return false;
    }

    /**
     * Load products for front end store
     * @param String $step
     * @return Array
     */
    private function getOffer($step)
    {
        $offerStep         = collect(Config::get('offer_details.' . $step));
        $insuranceProvider = [];
        $mainProducts      = [];
        $splitCharges      = [];

        if ($offerStep->isNotEmpty())
        {

            $selectedProduct = $offerStep->get('selectedProduct');

            if ($selectedProduct)
            {
                $mainProducts = $this->mapProductArgs($selectedProduct);
            }

            $splitChargeDetails = $offerStep->get('splitChargeDetails');

            if ($splitChargeDetails)
            {

                $splitCharges = $this->mapProductArgs($splitChargeDetails['selectedProduct']);

                if (!empty($splitChargeDetails['is_insurance_provider']))
                {
                    $insuranceProvider['isset'] = $splitChargeDetails['is_insurance_provider'];
                    $insuranceProvider['name']  = $splitChargeDetails['insurance_provider_name'];
                    $insuranceProvider['logo']  = $splitChargeDetails['insurance_provider_logo'];
                }
            }
        }

        return compact('mainProducts', 'splitCharges', 'insuranceProvider');
    }

    /**
     * Map funnel.json product keys with application level keys
     * @param Array $selectedProduct
     * @return Array
     */
    private function mapProductArgs(Array $selectedProduct)
    {
        $allowedProductArgsAs = [
            'product_id'                 => 'id',
            'product_name'               => 'name',
            'crm_product_price'          => 'price',
            'crm_product_qty'            => 'qty',
            'product_description'        => 'description',
            'original_image_src'         => 'img',
            'thumbnail_image_src'        => 'thumbnail'
        ];
        # Hide Shipping Word form Frontend:
        # Add extra field on a shipping profile called `DIGITAL_PRODUCT` with value `1` or `YES`.
        # After that you need to add this shipping profile to the funnel as a default shipping profile.
        if(empty(Config::get('selected_shipping_profile.custom_fields.DIGITAL_PRODUCT'))) {
            $allowedProductArgsAs['extra_fields.crm_product_shipping_id'] = 'shippingId';
            $allowedProductArgsAs['extra_fields.crm_product_shipping_price'] = 'shippingPrice';
        }

        $products = [];

        foreach ($selectedProduct as $product)
        {
            $p = [];
            foreach ($allowedProductArgsAs as $key => $value)
            {
                $default = "";
                if(in_array($value, [
                    "shippingPrice",
                    "price"
                ])) {
                    $default = 0;
                }

                $p[$value] = array_get($product, $key, $default);
            }
            $products[] = $p;
        }

        return $products;
    }

    /**
     * Prepare products from offer details (funnel.json)
     * @return Array
     */
    private function prepareProducts()
    {

        $allOffers     = [];
        $offer_details = Config::get('offer_details', []);

        if (count($offer_details) > 1)
        {
            foreach ($offer_details as $step => $value)
            {
                $allOffers[$step] = $this->getOffer($step);
            }
        }
        else
        {
            $allOffers['step1'] = isset($offer_details['step1']) ? $this->getOffer('step1') : [];
        }

        return $allOffers;
    }

}
