<?php

namespace Framework\Foundation;

use Framework\Foundation\Config;

class TemplateEngine
{

    protected $compiler;
    protected $loader;
    private $lexers = array(
        'tag_comment'   => array(
            '[#', '#]',
        ),
        'tag_block'     => array(
            '[%', '%]',
        ),
        'tag_variable'  => array(
            '[[', ']]',
        ),
        'interpolation' => array(
            '#[', ']',
        ),
    );

    /**
     * Setup engine loader and compiler
     * @param Array|String $loader
     */
    public function __construct($loader = null)
    {
        $this->loader = $loader;
    }

    public function setLoader($loader)
    {
        $this->loader = $loader;
    }

    public function templateLoaderFromArray(Array $arrayOfString)
    {

        $this->compiler = new \Twig_Environment(
                new \Twig_Loader_Array($arrayOfString)
        );
    }

    public function templateLoaderFromFiles($path = null)
    {
        $path = !empty($path) ? $path : Config::get('template_path');

        $this->compiler = new \Twig_Environment(new \Twig_Loader_Filesystem($path), array(
            'auto_reload' => true,
        ));

        // Registering custom twig filter start
        $this->compiler->addFilter(new \Twig_SimpleFilter(
                        'image', array(get_class($this), 'imageTagTwigFilter'), array(
                    'is_safe' => array('html'),
                        )
        ));
    }

    public function loadArray(Array $array, $tokens)
    {
        if (empty($widget))
            return [];

        foreach ($array as $key => $value)
        {
            $tokens[$key] = html_entity_decode($this->compiler()->render($key, $tokens));
        }

        return $tokens;
    }

    public function loadview($defaultPage, $token)
    {
        //First check if a page was set to load using custom routing
        if (Config::get('render_view'))
        {
            $defaultPage = Config::get('render_view') . ".html";
        }

        //Check if file exists
        $pagePath = BASE_PATH . DS . 'templates/';
        $pageName = str_replace('.html', '', $defaultPage);

        if (!file_exists($pagePath . $defaultPage))
        {
            //Look into example/ folder or Try by appending -example to name
            if (file_exists($pagePath . "examples/" . $pageName . ".html"))
            {
                $defaultPage = "examples/" . $pageName . ".html";
            }
            elseif (file_exists($pagePath . "forms/" . $pageName . ".html"))
            {
                $defaultPage = "forms/" . $pageName . ".html";
            }
            elseif (file_exists($pagePath . $pageName . "-example.html"))
            {
                $defaultPage = $pageName . "-example.html";
            }
        }

        //Redirect to 404 if page was not found
        $is_404 = false;
        if (empty($defaultPage) || !file_exists($pagePath . $defaultPage))
        {
            $is_404 = true;
            if (file_exists($pagePath . "404.html"))
            {
                $defaultPage = "404.html";
                http_response_code(404);
                header($_SERVER["SERVER_PROTOCOL"] . " 404 Not Found", true, 404);
            }
            elseif (file_exists($pagePath . "examples/404.html"))
            {
                $defaultPage = "examples/404.html";
                http_response_code(404);
                header($_SERVER["SERVER_PROTOCOL"] . " 404 Not Found", true, 404);
            }
            else
            {
                echo "404 File Not Found";
                http_response_code(404);
                header($_SERVER["SERVER_PROTOCOL"] . " 404 Not Found", true, 404);
            }
        }

        $output              = $this->compiler()->render($defaultPage, $token);
        $google_analytics_id = Config::get('funnel.google_analytics_id');
        if (!empty($google_analytics_id))
        {
            $output = preg_replace('#</body>#', PHP_EOL . $this->addAnalyticsScript($google_analytics_id) . PHP_EOL . '</body>', $output);
        }

        if ($is_404)
        {
            echo $output;
            die();
        }
        else
        {
            $protocol   = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
            $domainName = $_SERVER['HTTP_HOST'];
            $sitepath   = $protocol . $domainName;

            $sitepath = "."; //Override

            $script = "<script>var SITE_BASE_URL = '" . $sitepath . "';</script>";
            $output = str_replace('</head>', PHP_EOL . $script . PHP_EOL . '</head>', $output);

            return $output;
        }
    }

    public function compiler()
    {
        if (is_array($this->loader))
        {
            $this->templateLoaderFromArray($this->loader);
        }
        else
        {
            $this->templateLoaderFromFiles($this->loader);
        }

        // Registering custom twig filter end
        $lexer = new \Twig_Lexer($this->compiler, $this->lexers);

        $this->compiler->setLexer($lexer);

        return $this->compiler;
    }

    /**
     * Method for custom twig filter
     * @return String
     */
    private static function imageTagTwigFilter($string, $fontSize, $fontName, $hexColour)
    {
        return sprintf(
                '<img alt="%s" src="image?text=%s&fontSize=%s&fontName=%s&hexColour=%s"/>', $string, $string, $fontSize, $fontName, ltrim($hexColour, '#')
        );
    }

    private function addAnalyticsScript($analyticsId = '')
    {
        if (!empty($analyticsId))
        {
            return "<script>
                    var _gaq = _gaq || [];
                    _gaq.push(['_setAccount', '" . $analyticsId . "']);
                    _gaq.push(['_trackPageview']);

                    (function() {
                      var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
                      ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
                  var s = document.getElementsByTagName('script')[0];
                  s.parentNode.insertBefore(ga, s);
                  })()
                </script>";
        }

        return '';
    }

}
