<?php

namespace Framework\Modules\Crm\Providers;

use Framework\Foundation\Config;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use Framework\Modules\Crm\Contracts\Crm as CrmContracts;
use Framework\Foundation\Helper;

class Checkout extends Crm implements CrmContracts
{

    const DEFAULT_ERROR_MSG = "The request couldn't be completed! Please try again later";

    private static $responseCodes = array(
        10000 => 'APPROVED', // The request was successful.
        20000 => 'SOFT DECLINE', // The request was declined, though subsequent attempts may be successful
        30000 => 'HARD DECLINE', // The request was declined
        40000 => 'RISK RESPONSES', // The request triggered a risk response
    );

    public function __construct()
    {
        parent::__construct();
        $this->apiEndpoint   = Config::get('payment_provider.api_end_point');
        $this->customerData  = array();
        $this->apiPrivateKey = Config::get('payment_provider.api_password');
        $this->funnelId      = Helper::getFunnelId();
    }

    /**
     * New prospect(customer) create to elminara
     *
     * @param $formData -> prospect create form data
     * @return
     */
    public function newProspect($formData)
    {
        $data             = $this->prepareFormData($formData);
        $this->beforeAnyMethodCall($formData);
        $data['funnelId'] = $this->funnelId;
        $response         = $this->makeApiRequest(app()->container()->get('apiEndpoint') . '/transactions/customers', $data);
        if ($response->getStatusCode() !== 200)
        {
            return false;
        }
        $responseArray = json_decode($response->getBody()->getContents(), true);
        if (!$responseArray['success'])
        {
            $arrayResult = array(
                'status' => $responseArray['success'],
                'errors' => $responseArray['errors'],
            );
            return ['status' => false, 'api_response' => $arrayResult];
        }
        $arrayResult = array(
            'status'     => $responseArray['success'],
            'prospectId' => $responseArray['result']['customerId'],
        );
        return ['status' => true, 'api_response' => $arrayResult];
    }

    /**
     * Create order of prospect
     *
     * @param $formData -> form data for step 1 or 2
     * @return
     */
    public function newOrderWithProspect($formData)
    {
        $return   = [];
        $this->beforeAnyMethodCall($formData);
        $response = $this->performGatewayRequest();
        if (empty($response['success']))
        {
            return $this->checkResponse($response);
        }
        $data               = $this->productDataSet();
        $data['customerId'] = $formData['prospectId'];
        $data['funnelId']   = $this->funnelId;

        $data['billingSameAsShipping'] = $formData['billingSameAsShipping'];
        if ($formData['billingSameAsShipping'] == 'no')
        {
            $data['billingFirstName'] = $formData['billingFirstName'];
            $data['billingLastName']  = $formData['billingLastName'];
            $data['billingAddress']   = $this->getBillingDetailsForEl($formData);
        }
        $checkoutEndCheck = $response['response_code_range']; // 1 -> approved order, 2 -> soft decline, 3 -> hard decline order, 4 -> order with risk

        $data['transactionId']          = $response['api_response']['trackId'];
        $data['isDeclined']             = $checkoutEndCheck == 2 || $checkoutEndCheck == 3 ? 1 : 0;
        $data['transactionReferenceId'] = null;
        $data['gatewayResponse']        = json_encode($response['api_response']);
        $logResponse                    = $this->responseLog($checkoutEndCheck, $data, $formData, $response);

        //Approve or order with risk flag
        if ($checkoutEndCheck === 1 || $checkoutEndCheck === 4)
        {
            return $logResponse;
        }
        $return = array(
            'status'       => false,
            'api_response' => array(
                'response_message' => $response['response_message'],
                'is_declined'      => !empty($checkoutEndCheck) && in_array($checkoutEndCheck, [2, 3]) ? 1 : 0
        ));
        return $return;
    }

    /**
     * Create order with prospect
     *
     * @param $formData -> form data for step 1 or 2
     * @param $configData ->
     * @param $type ->
     * @return
     */
    public function newOrder($formData, $configData, $type)
    {
        $return   = [];
        $this->beforeAnyMethodCall($formData);
        $response = $this->performGatewayRequest();
        if (empty($response['success']))
        {
            return $this->checkResponse($response);
        }
        $data = $this->productDataSet();
        if (!empty($configData['customer_id']))
        {
            $data['customerId'] = $configData['customer_id'];
        }
        $data['funnelId']              = $this->funnelId;
        $data['firstName']             = $formData['firstName'];
        $data['lastName']              = $formData['lastName'];
        $data['phone']                 = $formData['phone'];
        $data['email']                 = $formData['email'];
        $data['address']               = array(
            'address1' => !empty($formData['address']) ? $formData['address'] : '',
            'address2' => !empty($formData['address2']) ? $formData['address2'] : '',
            'city'     => !empty($formData['city']) ? $formData['city'] : '',
            'country'  => !empty($formData['country']) ? $formData['country'] : '',
            'state'    => !empty($formData['state']) ? $formData['state'] : '',
            'zip'      => !empty($formData['zip']) ? $formData['zip'] : '',
        );
        $data['billingSameAsShipping'] = $formData['billingSameAsShipping'];
        if ($formData['billingSameAsShipping'] == 'no')
        {
            $data['billingAddress'] = $this->dataSet($formData);
        }
        if (!empty($formData['billingFirstName']) && !empty($formData['billingLastName']))
        {
            $data['billingFirstName'] = $formData['billingFirstName'];
            $data['billingLastName']  = $formData['billingLastName'];
        }
        $checkoutEndCheck = $response['response_code_range']; // 1 -> approved order, 2 -> soft decline, 3 -> hard decline order, 4 -> order with risk

        $data['transactionId']          = $response['api_response']['trackId'];
        $data['isDeclined']             = $checkoutEndCheck == 2 || $checkoutEndCheck == 3 ? 1 : 0;
        $data['transactionReferenceId'] = null;
        $data['gatewayResponse']        = json_encode($response['api_response']);

        $logResponse = $this->responseLog($checkoutEndCheck, $data, $formData, $response);

        if ($checkoutEndCheck === 1 || $checkoutEndCheck === 4)
        {
            return $logResponse;
        }

        $return = array(
            'status'       => false,
            'api_response' => array(
                'response_message' => $response['response_message'],
                'is_declined'      => !empty($checkoutEndCheck) && in_array($checkoutEndCheck, [2, 3]) ? 1 : 0
        ));
        return $return;
    }

    /**
     * Order data retrive from session
     *
     * @param $queryStrings ->
     * @param $orderIds -> order id
     * @return array
     */
    public function orderView($queryStrings, $orderIds)
    {
        $data = array();
        foreach ($orderIds as $orderId)
        {
            if (!empty($_SESSION['checkout_orders'][$orderId]))
            {
                $data[$orderId] = $_SESSION['checkout_orders'][$orderId];
            }
            else
            {
                $data[$orderId] = 'Order not found!';
            }
        }
        return $data;
    }

    /**
     * set the product data
     *
     * @return array
     */
    private function productDataSet()
    {
        $productDetails = $this->selectedProducts;
        if (!empty($productDetails) && count($productDetails) > 0)
        {
            $data['shippingId']    = $this->shippingProfile->get('shipping_profile_crm_id');
            $data['shippingPrice'] = $this->shippingProfile->get('shipping_profile_price');
            foreach ($productDetails as $key => $val)
            {
                $data['productDetails'][] = array(
                    'productId'       => !empty($val['product_id']) ? $val['product_id'] : '',
                    'productPrice'    => $val['crm_product_price'],
                    'productQuantity' => $val['crm_product_qty'],
                );
            }
        }
        return $data;
    }

    /**
     * arrange all details to make request
     *
     * @return response
     */
    private function performGatewayRequest()
    {
        $params   = array_replace_recursive(
                $this->getCustomerDataSet(),
                $this->getCardDetails(),
                $this->getBillingDetails(),
                $this->getShippingDetails(),
                $this->getPriceDetail()
        );
        $response = $this->makeCheckoutRequest($params);
        return $response;
    }

    /**
     * Get CUSTOMER detail
     *
     * @param
     * @return
     */
    public function getCustomerDataSet()
    {
        return array_filter(array(
            'autoCapture'          => 'n',
            'chargeMode'           => 1,
            'email'                => $this->customerData['email'],
            'customerName'         => sprintf('%s %s', $this->customerData['firstName'], $this->customerData['lastName']),
            'transactionIndicator' => '1',
            'trackId'              => uniqid(),
        ));
    }

    /**
     * Get shipping detail
     *
     * @param
     * @return
     */
    private function getShippingDetails()
    {
        return array_filter(array(
            'shippingDetails' => [
                'addressLine1' => $this->customerData['address'],
                'city'         => $this->customerData['city'],
                'state'        => $this->customerData['state'],
                'postcode'     => $this->customerData['zip'],
                'country'      => $this->customerData['country'],
                'phone'        => [
                    'number' => $this->customerData['phone'],
                ],
            ],
        ));
    }

    /**
     * Get billing detail
     *
     * @param
     * @return
     */
    private function getBillingDetails()
    {
        if (strtolower($this->customerData['billingSameAsShipping']) === 'no')
        {
            $data = array_filter([
                'addressLine1' => $this->customerData['billingAddress'],
                'postcode'     => $this->customerData['billingZip'],
                'country'      => $this->customerData['billingCountry'],
                'city'         => $this->customerData['billingCity'],
                'state'        => $this->customerData['billingState'],
                'phone'        => [
                    'number' => $this->customerData['phone'],
                ],
            ]);
            if (!empty($this->customerData['billingAddress2']))
            {
                $data['addressLine2'] = $formData['billingAddress2'];
            }
        }
        $data = array_filter([
            'addressLine1' => $this->customerData['address'],
            'postcode'     => $this->customerData['zip'],
            'country'      => $this->customerData['country'],
            'city'         => $this->customerData['city'],
            'state'        => $this->customerData['state'],
            'phone'        => [
                'number' => $this->customerData['phone'],
            ],
        ]);
        return $data;
    }

    /**
     * Get billing detail for el-minara order create
     *
     * @param
     * @return
     */
    private function getBillingDetailsForEl($formData)
    {
        $data = array_filter(array(
            "billingDetails" => [
                'address1' => $formData['billingAddress'],
                'city'     => $formData['billingCity'],
                'country'  => $formData['billingCountry'],
                'state'    => $formData['billingState'],
                'zip'      => $formData['billingZip'],
            ],
        ));
        if (!empty($formData['billingAddress2']))
        {
            $data['billingDetails']['address2'] = $formData['billingAddress2'];
        }
        return $data;
    }

    /**
     * Get card details
     *
     * @param
     * @return
     */
    private function getCardDetails()
    {
        return array_filter(array(
            "cardToken" => $this->customerData['cardToken'],
        ));
    }

    /**
     * Get amount and currency details
     *
     * @return array
     */
    private function getPriceDetail()
    {
        $value                  = 0;
        $valueMulBy             = 100;
        $data                   = array(
            'amount'   => 0.00,
            'shipping' => 0.00,
            'total'    => 0.00,
        );
        $priceDetails           = array(
            'value'    => 0,
            'currency' => Config::get('funnel.currency.symbol'),
        );
        $fullValueCurrencyCodes = array(
            'BYR', 'BIF', 'DJF', 'GNF', 'ISK', 'KMF', 'XAF', 'CLF', 'XPF', 'JPY', 'PYG', 'RWF', 'KRW', 'VUV', 'VND', 'XOF',
        );
        $mulBy1000CurrencyCodes = array(
            'BHD', 'LYD', 'JOD', 'KWD', 'OMR', 'TND',
        );
        if (!is_array($this->selectedProducts))
        {
            return $priceDetails;
        }
        foreach ($this->selectedProducts as $product)
        {
            if (!empty($product['crm_product_price']))
            {
                $data['amount'] += (float) $product['crm_product_price'];
            }
            if (!empty($product['crm_product_shipping_price']))
            {
                $data['shipping'] += (float) $product['crm_product_shipping_price'];
            }
            $data['total'] += (float) $data['amount'] + $data['shipping'];
        }
        $valueMulBy = in_array($priceDetails['currency'], $fullValueCurrencyCodes) ?
                1 :
                (in_array($priceDetails['currency'], $mulBy1000CurrencyCodes) ?
                1000 :
                100
                ); //1:full price, 1000:multiply by 1000, 100:multiply by 100

        $value = floor($data['total'] + 0.5) * $valueMulBy;
        return $priceDetails;
    }

    /**
     * Prepare the form data for customer create to elminara
     *
     * @param $formData -> form data for step 1 or 2
     * @return
     */
    private function prepareFormData($formData)
    {
        $data              = array();
        $data['firstName'] = $formData['firstName'];
        $data['lastName']  = $formData['lastName'];
        $data['phone']     = $formData['phone'];
        $data['email']     = $formData['email'];
        $data['address']   = array(
            'address1' => !empty($formData['address']) ? $formData['address'] : '',
            'address2' => !empty($formData['address2']) ? $formData['address2'] : '',
            'city'     => !empty($formData['city']) ? $formData['city'] : '',
            'country'  => !empty($formData['country']) ? $formData['country'] : '',
            'state'    => !empty($formData['state']) ? $formData['state'] : '',
            'zip'      => !empty($formData['zip']) ? $formData['zip'] : '',
        );
        return $data;
    }

    /**
     * Set the customer data
     *
     * @param $formData  -> to set form data
     * @return
     */
    private function beforeAnyMethodCall($formData)
    {
        if (!empty($_SESSION['prospect_shipping_data']))
        {
            $this->customerData = array_replace_recursive(
                    $this->customerData, $_SESSION['prospect_shipping_data']
            );
        }
        if (!empty($_SESSION['full_order_data']))
        {
            $this->customerData = array_replace_recursive(
                    $this->customerData, $_SESSION['full_order_data']
            );
        }

        $this->customerData = array_replace_recursive(
                $this->customerData, $formData
        );

        $this->response = array(
            'status'       => false,
            'api_response' => array(
                'response_message' => self::DEFAULT_ERROR_MSG,
            ),
        );
    }

    /**
     * Prepare the form data for http request
     *
     * @param $requestUrL -> url to make curl request
     * @param $formPostParams -> posted form data to make curl request
     * @return Object
     */
    protected function makeApiRequest($requestUrL, $formPostParams)
    {
        $response = $this->httpClient->post($requestUrL, [
            'body'    => json_encode($formPostParams),
            'headers' => [
                'Content-Type' => 'application/json',
                'Api-Token'    => Config::get('uni_api_token'),
            ],
        ]);
        return $response;
    }

    /**
     * make request to checkout
     *
     * @param $params -> all the details
     * @return array
     */
    private function makeCheckoutRequest($params)
    {
        $headers    = array(
            'Authorization' => $this->apiPrivateKey,
            'Content-Type'  => 'application/json',
            'Accept'        => 'application/json',
        );
        $url        = sprintf('%s/charges/token', rtrim($this->apiEndpoint, '/'));
        $httpClient = new Client(array('http_errors' => false));
        $httpClient->setDefaultOption('verify', false);

        try
        {
            $response = $httpClient->post($url, [
                'headers' => $headers,
                'body'    => json_encode($params),
            ]);
        }
        catch (ClientException $ex)
        {
            $response = $ex->getResponse();
        }
        $prearedResponse = $this->prepareCheckoutResponse($response);
        return $prearedResponse;
    }

    /**
     * prepare checkout response
     *
     * @param $response -> full response from gateway
     * @return array
     */
    private function prepareCheckoutResponse($response)
    {
        $return        = array();
        $responseCode  = $response->getStatusCode();
        $responseArray = json_decode($response->getBody(), 1);

        if ($responseCode !== 200)
        {
            $return = [
                'success'       => false,
                'error_code'    => $responseCode,
                'error_message' => $responseArray['message'],
                'api_exception' => $responseArray,
            ];
            return $return;
        }
        $responseRange = (int) ($responseArray['responseCode'] / 10000);

        if ($responseRange >= 1 && $responseRange <= 4)
        {
            $return = [
                'success'             => true,
                'response_code'       => $responseArray['responseCode'],
                'response_code_range' => $responseRange,
                'response_message'    => $responseArray['responseMessage'],
                'api_response'        => $responseArray,
            ];
        }
        return $return;
    }

    /**
     * check response
     *
     * @param $response -> filtered response of gateway
     * @return array
     */
    private function checkResponse($response)
    {
        $return = [
            'status'       => false,
            'api_response' => [
                'response_message' => self::DEFAULT_ERROR_MSG,
            ],
        ];
        if (empty($response['success']))
        {
            if (!empty($response['error_message']))
            {
                $return['api_response']['response_message'] = $response['error_message'];
            }
            if (!empty($response['api_exception']))
            {
                $return['api_response']['api_exception'] = $response['api_exception'];
            }
            return $return;
        }
    }

    /**
     * set the product data
     *
     * @param $checkoutEndCheck -> check if api response decline
     * @param $data -> product data set
     * @param $formData -> form data for step 1 or 2
     * @param $checkoutApiResponse -> all gateway response details
     * @return
     */
    private function responseLog($checkoutEndCheck, $data, $formData, $checkoutApiResponse)
    {
        $return   = array('status' => false, 'api_response' => self::DEFAULT_ERROR_MSG);
        $response = $this->makeApiRequest(app()->container()->get('apiEndpoint') . '/transactions/orders', $data);
        if ($response->getStatusCode() !== 200)
        {
            return false;
        }
        $dataSet       = $data;
        unset($dataSet['gatewayResponse']);
        $responseArray = json_decode($response->getBody()->getContents(), true);
        if (!empty($responseArray['success']))
        {
            $return['status']                                                 = true;
            $return['api_response']                                           = $responseArray['result'];
            $_SESSION['checkout_orders'][$responseArray['result']['orderId']] = $dataSet;
        }
        return $return;
    }

    public function isDecline($response)
    {
        return !empty($response['api_response']['is_declined']) ? true : false;
    }

    public function singleOrderView($formData)
    {
        
    }

    public function hasOwnedThisOrder($data, $formData)
    {
        
    }

    public function prepareDataForCancel($data, $formData)
    {
        
    }

    public function cancelOrder($formData)
    {
        
    }

}
