<?php

namespace Framework\Modules\Crm\Providers;

use Framework\Foundation\Config;
use Framework\Modules\Crm\Contracts\Crm as CrmContracts;
use GuzzleHttp\Client;

class Response extends Crm implements CrmContracts
{
    protected $api_base_url;
    protected $username;
    protected $password;
    protected $campaignId;
    protected $shippingId;
    protected $forceGatewayId;
    protected $selectedProducts;
    protected $ipAddress;
    protected $notes;
    protected $httpClient;
    protected $soapClient;
    protected $ApiGuid;
    protected static $membershipMethods = [
        'order_update',
        'order_find',
        'order_view',
        'prospect_update',
        'prospect_find',
        'prospect_view',
        'get_alternative_provider',
        'coupon_validate',
    ];
    protected static $formKeys          = [
        'CreateProspectEx'    => [
            'address'   => 'address1',
            'ipAddress' => 'ip',
        ],
        'CreateSubscription ' => [
            'address'    => 'address1',
            'cc_number'  => 'creditCard',
            'expmonth'   => 'expMonth',
            'expyear'    => 'expYear',
            'campaignId' => 'campaignID',
        ],
    ];
    protected static $cc_types          = [
        'amex'     => 1,
        'visa'     => 2,
        'master'   => 3,
        'discover' => 4,
        'mastercard' => 3
    ];

    public function __construct()
    {
        parent::__construct();
        $this->projectId = array_get($_SESSION, 'external_data.project_id', Config::get('offer_details.step1.triangle_project_id'));
        $this->ApiGuid   = config::get('payment_provider.api_guid');
    }

    protected function prepareEndpointApi($api_base_url, $method)
    {
        $endpointUrl = rtrim($api_base_url, '/') . "/api/v2/open/" . $method;
        return $endpointUrl;
    }

    protected function httpRequest($api, $new_form_params)
    {
        // Using Deconstructor to solve cURL error 2 , we need to do R&D for this
        $this->httpClient = null;
        $this->httpClient = new Client();
        $this->httpClient->setDefaultOption('verify', false);

        $response = $this->httpClient->post($api, [
            'body'    => json_encode($new_form_params),
            'headers' => ['Content-type' => 'application/json']]);

        return $response;
    }

    protected function callApi($method, $form_post_params, $selectedProduct = array())
    {
        $api = $this->prepareEndpointApi($this->api_base_url, $method);

        try
        {
            $response = $this->httpRequest($api, $form_post_params);

            if ($response->getStatusCode() == 200)
            {
                $apiResponse = json_decode($response->getBody()->getContents(), true);

                $status = (string) isset($apiResponse['Status']) ? $apiResponse['Status'] : '';
                if ($status === '')
                {
                    $apiResponseArr = ['response_message' => 'The request couldn\'t be completed! Please try again later'];
                    return ['status' => false, 'api_response' => $apiResponseArr];
                }

                if ($status != 0)
                {
                    $msg = isset($apiResponse['ErrorMessage'][0]) ? $apiResponse['ErrorMessage'][0] : 'The request couldn\'t be completed! Please try again later';

                    $apiResponseArr = ['response_message' => $msg];
                    return ['status' => false, 'api_response' => $apiResponseArr];
                }

                if (
                        !empty($apiResponse['Transaction']['OrderInfo']) &&
                        $apiResponse['Transaction']['OrderInfo']['Response'] != 1
                )
                {
                    return [
                        'status'       => false, 'api_response' => [
                            'response_message' => $apiResponse['Transaction']['OrderInfo']['ResponseText']
                        ]
                    ];
                }

                $prospectId = isset($apiResponse['CustomerID']) ? $apiResponse['CustomerID'] : '';
                $prospectId = isset($apiResponse['Transaction']['OrderInfo']['CustomerID']) ? $apiResponse['Transaction']['OrderInfo']['CustomerID'] : $prospectId;
                $orderId    = isset($apiResponse['Transaction']['OrderInfo']['TransactionID']) ? $apiResponse['Transaction']['OrderInfo']['TransactionID'] : '';

                if (empty($prospectId) && empty($orderId))
                {
                    $apiResponseArr = ['response_message' => 'The request couldn\'t be completed! Please try again later'];
                    return ['status' => false, 'api_response' => $apiResponseArr];
                }

                $apiResponseArr['prospectId'] = $prospectId;
                $apiResponseArr['customerId'] = $prospectId;
                if (!empty($orderId))
                {
                    $apiResponseArr['orderId']                = $orderId;
                    $apiResponse['selectedProducts']          = $selectedProduct;
                    $_SESSION['all_order'][(string) $orderId] = $apiResponse;
                }

                if ($method == 'transactions')
                {
                    $_SESSION['checkout'] = $apiResponse;
                }
                elseif ($method == 'customers')
                {
                    $_SESSION['prospect'] = $apiResponse;
                }
                elseif ($method == 'transactions/upsell')
                {
                    $_SESSION['upsell'] = $apiResponse;
                }

                return ['status' => true, 'api_response' => $apiResponseArr];
            }
            else
            {
                $apiResponseArr = ['response_message' => 'The request couldn\'t be completed! Please try again later'];
                return ['status' => false, 'api_response' => $apiResponseArr];
            }

            $apiResponseArr = ['response_message' => 'The request couldn\'t be completed! Please try again later'];
            return ['status' => false, 'api_response' => $apiResponseArr];
        }
        catch (\Exception $ex)
        {
            $apiResponseArr = ['response_message' => 'Something went wrong while processing the request'];
            return ['status' => false, 'api_response' => $apiResponseArr];
        }
    }

    public function newProspect($formData)
    {
        $method      = 'customers';
        $campaignIds = explode(',', $this->campaignId);
        if (count($campaignIds) > 1)
        {
            $campaignId = $campaignIds[0];
        }
        else
        {
            $campaignId = $this->campaignId;
        }
        $configData = array(
            'ApiGuid'        => $this->ApiGuid,
            'Siteid'         => $campaignId,
            'FirstName'      => isset($formData['firstName']) ? $formData['firstName'] : '',
            'LastName'       => isset($formData['lastName']) ? $formData['lastName'] : '',
            'Email'          => isset($formData['email']) ? $formData['email'] : '',
            'Phone'          => isset($formData['phone']) ? $formData['phone'] : '',
            'Address1'       => isset($formData['address']) ? $formData['address'] : '',
            'Address2'       => isset($formData['address2']) ? $formData['address2'] : '',
            'City'           => isset($formData['city']) ? $formData['city'] : '',
            'State'          => isset($formData['state']) ? $formData['state'] : '',
            'CountryIso'     => isset($formData['country']) ? $formData['country'] : '',
            'ZipCode'        => isset($formData['zip']) ? $formData['zip'] : '',
            'IpAddress'      => isset($this->ipAddress) ? $this->ipAddress : '',
            'AffiliateID'    => '',
            'SubAffiliateID' => '',
        );

        $crmData  = $configData;
        $response = $this->callApi($method, $crmData);

        return $response;
    }

    public function setDifferentBilling($data)
    {
        if (isset($data['billingSameAsShipping']) && (strtolower($data['billingSameAsShipping']) == 'no' || $data['billingSameAsShipping'] === false))
        {
            $replacements = [
                'billingFirstName' => 'firstName',
                'billingLastName'  => 'lastName',
                'billingAddress'   => 'address',
                'billingCity'      => 'city',
                'billingCountry'   => 'country',
                'billingState'     => 'state',
                'billingZip'       => 'zip',
            ];

            foreach ($replacements as $ri => $rv)
            {
                if (isset($data[$ri]) and!empty($data[$ri]))
                {
                    $data['shipping' . ucfirst($rv)] = $data[$rv];
                    //Replace new billing details
                    $data[$rv]                       = $data[$ri];
                }
            }
        }

        return $data;
    }

    public function newOrderWithProspect($checkoutData)
    {
        $method       = 'transactions';
        $checkoutData = array_merge($_SESSION['prospect_shipping_data'], $checkoutData);

        $checkoutData = $this->setDifferentBilling($checkoutData);
        $firstName    = isset($checkoutData['firstName']) ? $checkoutData['firstName'] : '';
        $lastName     = isset($checkoutData['lastName']) ? $checkoutData['lastName'] : '';
        $cardOnName   = isset($checkoutData['name_on_card']) ? $checkoutData['name_on_card'] : '';
        $cardOnName   = empty($cardOnName) ? ($firstName . ' ' . $lastName) : $cardOnName;
        $configData   = array(
            'ApiGuid'        => $this->ApiGuid,
            'CustomerID'     => $checkoutData['prospectId'],
            'IpAddress'      => isset($this->ipAddress) ? $this->ipAddress : '',
            'CouponCode'     => isset($checkoutData['promo_code']) ? $checkoutData['promo_code'] : '',
            'BillingAddress' => array(
                'FirstName'          => isset($checkoutData['firstName']) ? $checkoutData['firstName'] : '',
                'PaymentInformation' => array(
                    'LastName'   => isset($checkoutData['lastName']) ? $checkoutData['lastName'] : '',
                    'Address1'   => isset($checkoutData['address']) ? $checkoutData['address'] : '',
                    'Address2'   => isset($checkoutData['address2']) ? $checkoutData['address2'] : '',
                    'City'       => isset($checkoutData['city']) ? $checkoutData['city'] : '',
                    'State'      => isset($checkoutData['state']) ? $checkoutData['state'] : '',
                    'CountryIso' => isset($checkoutData['country']) ? $checkoutData['country'] : '',
                    'ZipCode'    => isset($checkoutData['zip']) ? $checkoutData['zip'] : '',
                ),
                'AffiliateID'        => '',
                'SubAffiliateID'     => '',
                'ExpMonth'           => $checkoutData['expmonth'],
                'ExpYear'            => (2000 + $checkoutData['expyear']),
                'CCNumber'           => $checkoutData['cc_number'],
                'NameOnCard'         => $cardOnName,
                'CVV'                => $checkoutData['cvv'],
                'ProductGroups'      => array(),
            ),
        );

        $configData['PaymentInformation']['ProductGroups'] = array();
        $productGroupKeys                                  = array();
        foreach ($this->selectedProducts as $product)
        {
            $productGroupKeys[] = isset($product['product_group_key']) ?
                    $product['product_group_key'] : "";
        }
        $productGroupKeys = array_values(array_unique($productGroupKeys));
        foreach ($productGroupKeys as $productGroupKey)
        {
            $configData['PaymentInformation']['ProductGroups'][] = array(
                'ProductGroupKey' => $productGroupKey,
                'CustomProducts'  => []
            );
        }

        foreach ($this->selectedProducts as $product)
        {
            foreach ($productGroupKeys as $key => $productGroupKey)
            {
                if (
                        isset($product['product_group_key']) &&
                        $product['product_group_key'] == $productGroupKey
                )
                {
                    $configData['PaymentInformation']['ProductGroups'][$key]['CustomProducts'][] = array(
                        'ProductID' => $product['crm_product_id'],
                        'Amount'    => $product['crm_product_price'],
                        'Quantity'  => $product['crm_product_qty'],
                    );
                }
            }
        }

        $crmData  = $configData;
        $response = $this->callApi($method, $crmData, $this->selectedProducts);

        return $response;
    }

    public function newOrder($orderData, $configData, $type)
    {
        //This config key will determe if a single page order will be sent in a breakdown of 2 steps (prospect+order, so as to differentiate billing and shipping address)

        if ($type == 'UpsellOrder')
        {
            $orderData = $this->setDifferentBilling($orderData);
            $response  = $this->upsell($orderData, $configData);
            return $response;
        }
        $configFull                = Config::get('payment_provider.full_order_force_prospect');
        $full_order_force_prospect = !empty($configFull) ? $configFull : 1;

        if ($full_order_force_prospect == 1)
        {
            $prospect_response = $this->newProspect($orderData);
            if (!isset($prospect_response['status']) or!$prospect_response['status'] or!isset($prospect_response['api_response']['prospectId']))
            {
                return $prospect_response;
            }
            $prospect_id = $prospect_response['api_response']['prospectId'];

            if (!isset($_SESSION['full_order_data_copy']))
            {
                $_SESSION['full_order_data_copy'] = [];
            }

            $_SESSION['full_order_data_copy']['prospectId'] = $prospect_id;
            $orderData['prospectId']                        = $prospect_id;
        }

        if (isset($_SESSION['prospect_shipping_data']))
        {
            $orderData = array_merge($_SESSION['prospect_shipping_data'], $orderData);
        }
        if (isset($_SESSION['full_order_data_copy']))
        {
            $session = (object) $_SESSION['full_order_data_copy'];
        }

        $orderData              = $this->setDifferentBilling($orderData);
        $this->selectedProducts = isset($configData['selectedProducts']) ? $configData['selectedProducts'] : $this->selectedProducts;
        $response               = $this->newOrderWithProspect($orderData);
        return $response;
    }

    public function orderView($queryStrings, $orderIds)
    {
        $apiResponseArr = [];
        $fullOrder      = isset($_SESSION['full_order_data_copy']) ? $_SESSION['full_order_data_copy'] : [];
        $checkoutDetail = isset($_SESSION['checkout']) ? $_SESSION['checkout'] : [];
        $allOrderDetail = isset($_SESSION['all_order']) ? $_SESSION['all_order'] : [];
        $bill_same      = isset($fullOrder['billingSameAsShipping']) && $fullOrder['billingSameAsShipping'] == 'no' ? false : true;

        if (empty($allOrderDetail))
        {
            $apiResponseArr['acquisition_date']                 = date('d-m-Y');
            $apiResponseArr['ancestor_id']                      = isset($checkoutDetail['Transaction']['OrderInfo']['TransactionID']) ? $checkoutDetail['Transaction']['OrderInfo']['TransactionID'] : '';
            $apiResponseArr['affiliate']                        = '';
            $apiResponseArr['afid']                             = '';
            $apiResponseArr['sid']                              = '';
            $apiResponseArr['affid']                            = '';
            $apiResponseArr['c1']                               = '';
            $apiResponseArr['c2']                               = '';
            $apiResponseArr['c3']                               = '';
            $apiResponseArr['aid']                              = '';
            $apiResponseArr['opt']                              = '';
            $apiResponseArr['amount_refunded_to_date']          = isset($checkoutDetail['Transaction']['OrderInfo']['TotalAmount']) ? $checkoutDetail['Transaction']['OrderInfo']['TotalAmount'] : '';
            $apiResponseArr['auth_id']                          = '';
            $apiResponseArr['billing_city']                     = $bill_same ? (isset($fullOrder['city']) ? $fullOrder['city'] : '') : (isset($fullOrder['billingCity']) ? $fullOrder['billingCity'] : '');
            $apiResponseArr['billing_country']                  = $bill_same ? (isset($fullOrder['country']) ? $fullOrder['country'] : '') : (isset($fullOrder['billingCountry']) ? $fullOrder['billingCountry'] : '');
            $apiResponseArr['billing_cycle']                    = '';
            $apiResponseArr['billing_first_name']               = $bill_same ? (isset($fullOrder['firstName']) ? $fullOrder['firstName'] : '') : (isset($fullOrder['billingFirstName']) ? $fullOrder['billingFirstName'] : '');
            $apiResponseArr['billing_last_name']                = $bill_same ? (isset($fullOrder['lastName']) ? $fullOrder['lastName'] : '') : (isset($fullOrder['billingLastName']) ? $fullOrder['billingLastName'] : '');
            $apiResponseArr['billing_postcode']                 = $bill_same ? (isset($fullOrder['zip']) ? $fullOrder['zip'] : '') : (isset($fullOrder['billingZip']) ? $fullOrder['billingZip'] : '');
            $apiResponseArr['billing_state']                    = $bill_same ? (isset($fullOrder['state']) ? $fullOrder['state'] : '') : (isset($fullOrder['billingState']) ? $fullOrder['billingState'] : '');
            $apiResponseArr['billing_state_id']                 = $bill_same ? (isset($fullOrder['state']) ? $fullOrder['state'] : '') : (isset($fullOrder['billingState']) ? $fullOrder['billingState'] : '');
            $apiResponseArr['billing_street_address']           = $bill_same ? (isset($fullOrder['address']) ? $fullOrder['address'] : '') : (isset($fullOrder['billingAddress']) ? $fullOrder['billingAddress'] : '');
            $apiResponseArr['billing_street_address2']          = '';
            $apiResponseArr['cc_expires']                       = isset($fullOrder['expmonth']) ? $fullOrder['expmonth'] : '' . "-" . 2000 + (isset($fullOrder['expyear']) ? $fullOrder['expyear'] : 0);
            $apiResponseArr['cc_first_6']                       = substr(isset($fullOrder['cc_number']) ? $fullOrder['cc_number'] : '1111111111111111', 0, 6);
            $apiResponseArr['cc_last_4']                        = substr(isset($fullOrder['cc_number']) ? $fullOrder['cc_number'] : '1111111111111111', 0, -4);
            $apiResponseArr['cc_number']                        = isset($fullOrder['cc_number']) ? $fullOrder['cc_number'] : '1111111111111111';
            $apiResponseArr['credit_card_number']               = isset($fullOrder['cc_number']) ? $fullOrder['cc_number'] : '1111111111111111';
            $apiResponseArr['cc_type']                          = isset($fullOrder['cc_type']) ? $fullOrder['cc_type'] : '1111111111111111';
            $apiResponseArr['chargeback_date']                  = '';
            $apiResponseArr['check_account_last_4']             = '';
            $apiResponseArr['check_routing_last_4']             = '';
            $apiResponseArr['check_ssn_last_4']                 = '';
            $apiResponseArr['check_transitnum']                 = '';
            $apiResponseArr['child_id']                         = '';
            $apiResponseArr['click_id']                         = '';
            $apiResponseArr['created_by_user_name']             = '';
            $apiResponseArr['created_by_employee_name']         = '';
            $apiResponseArr['coupon_discount_amount']           = '';
            $apiResponseArr['customer_id']                      = '';
            $apiResponseArr['customers_telephone']              = isset($fullOrder['phone']) ? $fullOrder['phone'] : '';
            $apiResponseArr['decline_salvage_discount_percent'] = '';
            $apiResponseArr['decline_reason']                   = '';
            $apiResponseArr['email_address']                    = isset($fullOrder['email']) ? $fullOrder['email'] : '';
            $apiResponseArr['first_name']                       = isset($fullOrder['firstName']) ? $fullOrder['firstName'] : '';
            $apiResponseArr['gateway_id']                       = '';
            $apiResponseArr['gateway_descriptor']               = '';
            $apiResponseArr['hold_date']                        = '';
            $apiResponseArr['ip_address']                       = $this->ipAddress;
            $apiResponseArr['is_blacklisted']                   = '';
            $apiResponseArr['is_chargeback']                    = '';
            $apiResponseArr['is_fraud']                         = '';
            $apiResponseArr['is_recurring']                     = '';
            $apiResponseArr['is_refund']                        = '';
            $apiResponseArr['is_rma']                           = '';
            $apiResponseArr['is_test_cc']                       = '';
            $apiResponseArr['is_void']                          = '';
            $apiResponseArr['last_name']                        = isset($fullOrder['lastName']) ? $fullOrder['lastName'] : '';
            $apiResponseArr['prepaid_match']                    = '';
            $apiResponseArr['preserve_gateway']                 = '';
            $apiResponseArr['processor_id']                     = '';
            $apiResponseArr['rebill_discount_percent']          = '0.00';
            $apiResponseArr['recurring_date']                   = '';
            $apiResponseArr['refund_amount']                    = '';
            $apiResponseArr['refund_date']                      = '';
            $apiResponseArr['retry_date']                       = '';
            $apiResponseArr['rma_number']                       = '';
            $apiResponseArr['rma_reason']                       = '';
            $apiResponseArr['shipping_city']                    = isset($fullOrder['city']) ? $fullOrder['city'] : '';
            $apiResponseArr['shipping_country']                 = isset($fullOrder['country']) ? $fullOrder['country'] : '';
            $apiResponseArr['shipping_date']                    = '';
            $apiResponseArr['shipping_first_name']              = isset($fullOrder['firstName']) ? $fullOrder['firstName'] : '';
            $apiResponseArr['shipping_id']                      = '';
            $apiResponseArr['shipping_last_name']               = isset($fullOrder['lastName']) ? $fullOrder['lastName'] : '';
            $apiResponseArr['shipping_method_name']             = '';
            $apiResponseArr['shipping_postcode']                = isset($fullOrder['zip']) ? $fullOrder['zip'] : '';
            $apiResponseArr['shipping_state']                   = isset($fullOrder['state']) ? $fullOrder['state'] : '';
            $apiResponseArr['shipping_state_id']                = isset($fullOrder['state']) ? $fullOrder['state'] : '';
            $apiResponseArr['shipping_street_address']          = isset($fullOrder['address']) ? $fullOrder['address'] : '';
            $apiResponseArr['shipping_street_address2']         = '';
            $apiResponseArr['sub_affiliate']                    = '';
            $apiResponseArr['time_stamp']                       = '';
            $apiResponseArr['tracking_number']                  = isset($checkoutDetail['Transaction']['OrderInfo']['TransactionID']) ? $checkoutDetail['Transaction']['OrderInfo']['TransactionID'] : '';
            $apiResponseArr['transaction_id']                   = isset($checkoutDetail['Transaction']['OrderInfo']['TransactionID']) ? $checkoutDetail['Transaction']['OrderInfo']['TransactionID'] : '';
            $apiResponseArr['upsell_product_id']                = '';
            $apiResponseArr['upsell_product_quantity']          = '';
            $apiResponseArr['void_amount']                      = '';
            $apiResponseArr['void_date']                        = '';
            $apiResponseArr['shippable']                        = '';
            $apiResponseArr['systemNotes']                      = '';
            $apiResponseArr['total_price']                      = isset($checkoutDetail['Transaction']['OrderInfo']['TotalAmount']) ? $checkoutDetail['Transaction']['OrderInfo']['TotalAmount'] : '';
            $apiResponseArr['shippingPrice']                    = isset($checkoutDetail['Transaction']['OrderInfo']['ShippingAmount']) ? $checkoutDetail['Transaction']['OrderInfo']['ShippingAmount'] : '';
            $apiResponseArr['sales_tax']                        = isset($checkoutDetail['Transaction']['OrderInfo']['SalesTax']) ? $checkoutDetail['Transaction']['OrderInfo']['SalesTax'] : '';

            $apiResponseArr['products'] = isset($this->selectedProducts) ? $this->selectedProducts : [];
        }
        else
        {
            foreach ($allOrderDetail as $key => $val)
            {

                $apiResponseArr[$key]['acquisition_date']                 = date('d-m-Y');
                $apiResponseArr[$key]['ancestor_id']                      = isset($val['Transaction']['OrderInfo']['TransactionID']) ? $checkoutDetail['Transaction']['OrderInfo']['TransactionID'] : '';
                $apiResponseArr[$key]['affiliate']                        = '';
                $apiResponseArr[$key]['afid']                             = '';
                $apiResponseArr[$key]['sid']                              = '';
                $apiResponseArr[$key]['affid']                            = '';
                $apiResponseArr[$key]['c1']                               = '';
                $apiResponseArr[$key]['c2']                               = '';
                $apiResponseArr[$key]['c3']                               = '';
                $apiResponseArr[$key]['aid']                              = '';
                $apiResponseArr[$key]['opt']                              = '';
                $apiResponseArr[$key]['amount_refunded_to_date']          = isset($checkoutDetail['Transaction']['OrderInfo']['TotalAmount']) ? $checkoutDetail['Transaction']['OrderInfo']['TotalAmount'] : '';
                $apiResponseArr[$key]['auth_id']                          = '';
                $apiResponseArr[$key]['billing_city']                     = $bill_same ? (isset($fullOrder['city']) ? $fullOrder['city'] : '') : (isset($fullOrder['billingCity']) ? $fullOrder['billingCity'] : '');
                $apiResponseArr[$key]['billing_country']                  = $bill_same ? (isset($fullOrder['country']) ? $fullOrder['country'] : '') : (isset($fullOrder['billingCountry']) ? $fullOrder['billingCountry'] : '');
                $apiResponseArr[$key]['billing_cycle']                    = '';
                $apiResponseArr[$key]['billing_first_name']               = $bill_same ? (isset($fullOrder['firstName']) ? $fullOrder['firstName'] : '') : (isset($fullOrder['billingFirstName']) ? $fullOrder['billingFirstName'] : '');
                $apiResponseArr[$key]['billing_last_name']                = $bill_same ? (isset($fullOrder['lastName']) ? $fullOrder['lastName'] : '') : (isset($fullOrder['billingLastName']) ? $fullOrder['billingLastName'] : '');
                $apiResponseArr[$key]['billing_postcode']                 = $bill_same ? (isset($fullOrder['zip']) ? $fullOrder['zip'] : '') : (isset($fullOrder['billingZip']) ? $fullOrder['billingZip'] : '');
                $apiResponseArr[$key]['billing_state']                    = $bill_same ? (isset($fullOrder['state']) ? $fullOrder['state'] : '') : (isset($fullOrder['billingState']) ? $fullOrder['billingState'] : '');
                $apiResponseArr[$key]['billing_state_id']                 = $bill_same ? (isset($fullOrder['state']) ? $fullOrder['state'] : '') : (isset($fullOrder['billingState']) ? $fullOrder['billingState'] : '');
                $apiResponseArr[$key]['billing_street_address']           = $bill_same ? (isset($fullOrder['address']) ? $fullOrder['address'] : '') : (isset($fullOrder['billingAddress']) ? $fullOrder['billingAddress'] : '');
                $apiResponseArr[$key]['billing_street_address2']          = '';
                $apiResponseArr[$key]['cc_expires']                       = isset($fullOrder['expmonth']) ? $fullOrder['expmonth'] : '' . "-" . 2000 + (isset($fullOrder['expyear']) ? $fullOrder['expyear'] : 0);
                $apiResponseArr[$key]['cc_first_6']                       = substr(isset($fullOrder['cc_number']) ? $fullOrder['cc_number'] : '1111111111111111', 0, 6);
                $apiResponseArr[$key]['cc_last_4']                        = substr(isset($fullOrder['cc_number']) ? $fullOrder['cc_number'] : '1111111111111111', 0, -4);
                $apiResponseArr[$key]['cc_number']                        = isset($fullOrder['cc_number']) ? $fullOrder['cc_number'] : '1111111111111111';
                $apiResponseArr[$key]['credit_card_number']               = isset($fullOrder['cc_number']) ? $fullOrder['cc_number'] : '1111111111111111';
                $apiResponseArr[$key]['cc_type']                          = isset($fullOrder['cc_type']) ? $fullOrder['cc_type'] : '1111111111111111';
                $apiResponseArr[$key]['chargeback_date']                  = '';
                $apiResponseArr[$key]['check_account_last_4']             = '';
                $apiResponseArr[$key]['check_routing_last_4']             = '';
                $apiResponseArr[$key]['check_ssn_last_4']                 = '';
                $apiResponseArr[$key]['check_transitnum']                 = '';
                $apiResponseArr[$key]['child_id']                         = '';
                $apiResponseArr[$key]['click_id']                         = '';
                $apiResponseArr[$key]['created_by_user_name']             = '';
                $apiResponseArr[$key]['created_by_employee_name']         = '';
                $apiResponseArr[$key]['coupon_discount_amount']           = '';
                $apiResponseArr[$key]['customer_id']                      = '';
                $apiResponseArr[$key]['customers_telephone']              = isset($fullOrder['phone']) ? $fullOrder['phone'] : '';
                $apiResponseArr[$key]['decline_salvage_discount_percent'] = '';
                $apiResponseArr[$key]['decline_reason']                   = '';
                $apiResponseArr[$key]['email_address']                    = isset($fullOrder['email']) ? $fullOrder['email'] : '';
                $apiResponseArr[$key]['first_name']                       = isset($fullOrder['firstName']) ? $fullOrder['firstName'] : '';
                $apiResponseArr[$key]['gateway_id']                       = '';
                $apiResponseArr[$key]['gateway_descriptor']               = '';
                $apiResponseArr[$key]['hold_date']                        = '';
                $apiResponseArr[$key]['ip_address']                       = $this->ipAddress;
                $apiResponseArr[$key]['is_blacklisted']                   = '';
                $apiResponseArr[$key]['is_chargeback']                    = '';
                $apiResponseArr[$key]['is_fraud']                         = '';
                $apiResponseArr[$key]['is_recurring']                     = '';
                $apiResponseArr[$key]['is_refund']                        = '';
                $apiResponseArr[$key]['is_rma']                           = '';
                $apiResponseArr[$key]['is_test_cc']                       = '';
                $apiResponseArr[$key]['is_void']                          = '';
                $apiResponseArr[$key]['last_name']                        = isset($fullOrder['lastName']) ? $fullOrder['lastName'] : '';
                $apiResponseArr[$key]['prepaid_match']                    = '';
                $apiResponseArr[$key]['preserve_gateway']                 = '';
                $apiResponseArr[$key]['processor_id']                     = '';
                $apiResponseArr['rebill_discount_percent']                = '0.00';
                $apiResponseArr[$key]['recurring_date']                   = '';
                $apiResponseArr[$key]['refund_amount']                    = '';
                $apiResponseArr[$key]['refund_date']                      = '';
                $apiResponseArr[$key]['retry_date']                       = '';
                $apiResponseArr[$key]['rma_number']                       = '';
                $apiResponseArr[$key]['rma_reason']                       = '';
                $apiResponseArr[$key]['shipping_city']                    = isset($fullOrder['city']) ? $fullOrder['city'] : '';
                $apiResponseArr[$key]['shipping_country']                 = isset($fullOrder['country']) ? $fullOrder['country'] : '';
                $apiResponseArr[$key]['shipping_date']                    = '';
                $apiResponseArr[$key]['shipping_first_name']              = isset($fullOrder['firstName']) ? $fullOrder['firstName'] : '';
                $apiResponseArr[$key]['shipping_id']                      = ''; //$PrimaryOrder->BillingID;
                $apiResponseArr[$key]['shipping_last_name']               = isset($fullOrder['lastName']) ? $fullOrder['lastName'] : '';
                $apiResponseArr[$key]['shipping_method_name']             = '';
                $apiResponseArr[$key]['shipping_postcode']                = isset($fullOrder['zip']) ? $fullOrder['zip'] : '';
                $apiResponseArr[$key]['shipping_state']                   = isset($fullOrder['state']) ? $fullOrder['state'] : '';
                $apiResponseArr[$key]['shipping_state_id']                = isset($fullOrder['state']) ? $fullOrder['state'] : '';
                $apiResponseArr[$key]['shipping_street_address']          = isset($fullOrder['address']) ? $fullOrder['address'] : '';
                $apiResponseArr[$key]['shipping_street_address2']         = '';
                $apiResponseArr[$key]['sub_affiliate']                    = '';
                $apiResponseArr[$key]['time_stamp']                       = '';
                $apiResponseArr[$key]['tracking_number']                  = isset($val['Transaction']['OrderInfo']['TransactionID']) ? $val['Transaction']['OrderInfo']['TransactionID'] : '';
                $apiResponseArr[$key]['transaction_id']                   = isset($val['Transaction']['OrderInfo']['TransactionID']) ? $val['Transaction']['OrderInfo']['TransactionID'] : '';
                $apiResponseArr[$key]['upsell_product_id']                = '';
                $apiResponseArr[$key]['upsell_product_quantity']          = '';
                $apiResponseArr[$key]['void_amount']                      = '';
                $apiResponseArr[$key]['void_date']                        = '';
                $apiResponseArr[$key]['shippable']                        = '';
                $apiResponseArr[$key]['systemNotes']                      = '';
                $apiResponseArr[$key]['total_price']                      = isset($val['Transaction']['OrderInfo']['TotalAmount']) ? $val['Transaction']['OrderInfo']['TotalAmount'] : '';
                $apiResponseArr[$key]['order_total']                      = isset($val['Transaction']['OrderInfo']['TotalAmount']) ? $val['Transaction']['OrderInfo']['TotalAmount'] : '';

                $apiResponseArr[$key]['shippingPrice'] = isset($val['Transaction']['OrderInfo']['ShippingAmount']) ? $val['Transaction']['OrderInfo']['ShippingAmount'] : '';
                $apiResponseArr['sales_tax']           = isset($val['Transaction']['OrderInfo']['SalesTax']) ? $val['Transaction']['OrderInfo']['SalesTax'] : '';
                $selProduct                            = isset($val['selectedProducts']) ? $val['selectedProducts'] : [];
                foreach ($selProduct as $productKey => $productVal)
                {
                    $apiResponseArr[$key]['products'][]                                                  = array
                        (
                        'product_id'                   => isset($productVal['crm_product_id']) ? $productVal['crm_product_id'] : '',
                        'sku'                          => '',
                        'price'                        => isset($productVal['crm_product_price']) ? $productVal['crm_product_price'] : '',
                        'product_qty'                  => isset($productVal['crm_product_qty']) ? $productVal['crm_product_qty'] : '',
                        'name'                         => isset($productVal['product_name']) ? $productVal['product_name'] : '',
                        'on_hold'                      => 0,
                        'is_recurring'                 => '',
                        'recurring_date'               => date('Y-m-d'),
                        'subscription_id'              => '',
                        'next_subscription_product'    => '',
                        'next_subscription_product_id' => '',
                        'subscription_type'            => '',
                        'subscription_desc'            => '',
                    );
                    $apiResponseArr[$key]['products[' . $productKey . '][product_id]']                   = isset($productVal['crm_product_id']) ? $productVal['crm_product_id'] : '';
                    $apiResponseArr[$key]['products[' . $productKey . '][sku]']                          = '';
                    $apiResponseArr[$key]['products[' . $productKey . '][price]']                        = isset($productVal['crm_product_price']) ? $productVal['crm_product_price'] : '';
                    $apiResponseArr[$key]['products[' . $productKey . '][product_qty]']                  = isset($productVal['product_qty']) ? $productVal['product_qty'] : '';
                    $apiResponseArr[$key]['products[' . $productKey . '][product_qty]']                  = isset($productVal['crm_product_qty']) ? $productVal['crm_product_qty'] : '';
                    $apiResponseArr[$key]['products[' . $productKey . '][product_qty]']                  = isset($productVal['crm_product_qty']) ? $productVal['crm_product_qty'] : '';
                    $apiResponseArr[$key]['products[' . $productKey . '][on_hold]']                      = '';
                    $apiResponseArr[$key]['products[' . $productKey . '][is_recurring]']                 = '';
                    $apiResponseArr[$key]['products[' . $productKey . '][recurring_date]']               = date('Y-m-d');
                    $apiResponseArr[$key]['products[' . $productKey . '][subscription_id]']              = '';
                    $apiResponseArr[$key]['products[' . $productKey . '][next_subscription_product]']    = '';
                    $apiResponseArr[$key]['products[' . $productKey . '][next_subscription_product_id]'] = '';
                    $apiResponseArr[$key]['products[' . $productKey . '][subscription_type]']            = '';
                    $apiResponseArr[$key]['products[' . $productKey . '][subscription_desc]']            = '';
                }
            }
        }

        return $apiResponseArr;
    }

    public function upsell($orderData, $productConfig)
    {
        $method          = 'transactions/upsell';
        $configData      = array(
            'ApiGuid'       => $this->ApiGuid,
            'CustomerID'    => isset($orderData['prospect_id']) ? $orderData['prospect_id'] : '',
            'IpAddress'     => isset($this->ipAddress) ? $this->ipAddress : '',
            'ProductGroups' => array(),
        );
        $selectedProduct = isset($productConfig['selectedProducts']) ? $productConfig['selectedProducts'] : [];

        foreach ($selectedProduct as $key => $val)
        {
            $configData['ProductGroups'][] = ['ProductGroupKey' => $val['product_group_key']];
        }
        $crmData = $configData;

        $response = $this->callApi($method, $configData, $selectedProduct);
        return $response;
    }

    public function isDecline($response)
    {
        return empty($response['api_response']['orderId']) ? true : false;
    }

    public function singleOrderView($formData)
    {
        
    }

    public function hasOwnedThisOrder($data, $formData)
    {
        
    }

    public function prepareDataForCancel($data, $formData)
    {
        
    }

    public function cancelOrder($formData)
    {
        
    }

    public function validateCoupon($couponData)
    {
        try
        {
            $couponCode = $couponData['promo_code'];
            $method     = 'sites/' . $this->campaignId . '/coupons/' . $couponCode;
            $configData = array('ApiGuid' => $this->ApiGuid);
            $api        = $this->prepareEndpointApi($this->api_base_url, $method);

            $response = $this->httpClient->get($api, [
                'body'    => json_encode($configData),
                'headers' => ['Content-type' => 'application/json']
            ]);

            if ($response->getStatusCode() == 200)
            {
                $apiResponse = json_decode($response->getBody()->getContents(), true);
                $status      = isset($apiResponse['Status']) ? $apiResponse['Status'] : '';
                if ($status == 0)
                {
                    return ['status' => true, 'api_response' => $apiResponse];
                }
                $apiResponseArr = isset($apiResponse['ErrorMessage']) ? $apiResponse['ErrorMessage'] : '';
                return ['status' => false, 'api_response' => $apiResponseArr];
            }
            else
            {
                $apiResponseArr = ['response_message' => 'The request couldn\'t be completed! Please try again later'];
                return ['status' => false, 'api_response' => $apiResponseArr];
            }
        }
        catch (\Exception $ex)
        {
            throw new \Exception($ex->getMessage());
        }
    }

}
