<?php

namespace Framework\Modules\Crm\Providers;

use Framework\Foundation\Config;
use Framework\Modules\Crm\Contracts\Crm as CrmContracts;

class Square extends Crm implements CrmContracts
{

    const ERROR_RESPONSE = "The request couldn't be completed! Please try again later";

    public function __construct()
    {
        parent::__construct();
        $this->location_id   = Config::get('payment_provider.sq_location_id');
        $this->currency_code = Config::get('funnel.currency.code');
    }

    private function callApiRequest($method, $param)
    {
        $ch        = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->api_base_url . $method);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($param));
        $headers   = array();
        $headers[] = 'Content-Type: application/json';
        $headers[] = 'Square-Version: 2020-02-26';
        $headers[] = 'Authorization: Bearer ' . $this->username;
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        if (curl_errno($ch))
        {
            echo 'Error:' . curl_error($ch);
        }
        curl_close($ch);

        return json_decode($result, 1);
    }

    private function getIdempotency()
    {

        $permitted_chars = '0123456789abcdefghijklmnopqrstuvwxyz';
        return substr(str_shuffle($permitted_chars), 0, 10);
    }

    public function newProspect($formData)
    {
        $status      = false;
        $arrayResult = array(
            'status' => false,
            'errors' => 'Something went wrong please try again!',
        );

        $customerDetails = [
            "idempotency_key" => $this->getIdempotency(),
            "given_name"      => $formData['firstName'],
            "family_name"     => $formData['lastName'],
            "email_address"   => $formData['email'],
            "address"         => [
                "address_line_1"                  => $formData['address'],
                "locality"                        => $formData['city'],
                "administrative_district_level_1" => $formData['state'],
                "postal_code"                     => $formData['zip'],
                "country"                         => $formData['country']
            ],
            "phone_number"    => $formData['phone']
        ];

        $response = $this->callApiRequest('customers', $customerDetails);

        if (!empty($response) && isset($response['customer']))
        {
            $status      = true;
            $arrayResult = array(
                'status'     => true,
                'prospectId' => $response['customer']['id'],
            );
        }
        else if (!empty($response) && isset($response['errors']) && !empty($response['errors'][0]['detail']))
        {
            $status      = false;
            $arrayResult = array(
                'status' => false,
                'errors' => $response['errors'][0]['detail'],
            );
        }

        return ['status' => $status, 'api_response' => $arrayResult];
    }

    public function newOrderWithProspect($formData)
    {
        try
        {
            $status      = false;
            $arrayResult = array(
                'status' => false,
                'errors' => 'Something went wrong please try again!',
            );
            $this->beforeAnyMethodCall($formData);

            $orderId = $this->createOrder($formData);

            return $this->createPayment($formData, $orderId);
        }
        catch (\Exception $ex)
        {
            $arrayResult['errors'] = ($ex->getCode() == 9999) ? $ex->getMessage() : $arrayResult['errors'];
            return ["status" => false, "api_response" => $arrayResult];
        }
    }

    public function newOrder($formData, $configData, $type)
    {
        $arrayResult = array(
            'status' => false,
            'errors' => 'Something went wrong please try again!',
        );
        try
        {

            $this->beforeAnyMethodCall($formData);

            $prospect = $this->newProspect($formData);
            if ($prospect['status'])
            {
                $formData['prospectId'] = $prospect['api_response']['prospectId'];
                $orderId                = $this->createOrder($formData);

                return $this->createPayment($formData, $orderId);
            }
            return ["status" => false, "api_response" => $arrayResult];
        }
        catch (\Exception $ex)
        {
            $arrayResult['errors'] = ($ex->getCode() == 9999) ? $ex->getMessage() : $arrayResult['errors'];
            return ["status" => false, "api_response" => $arrayResult];
        }
    }

    private function createOrder($formData)
    {

        $items = [];
        if (!empty($this->selectedProducts))
        {
            $shipPrice = 0;
            foreach ($this->selectedProducts as $key => $value)
            {

                $items[]   = [
                    "uid"              => (string) $value['product_id'],
                    "name"             => (!empty($value['product_name']) ? $value['product_name'] : "Item :" . ($key + 1)),
                    "quantity"         => (string) $value['crm_product_qty'],
                    "base_price_money" => [
                        "amount"   => (int) round($value['crm_product_price'] * $value['crm_product_qty']),
                        "currency" => $this->currency_code
                    ]
                ];
                $shipPrice += $value['crm_product_shipping_price'];
            }
        }

        // Set shipping price from main shipping profile if individual product shipping price is empty (added on Jul 6 2021)
        if(empty($shipPrice) && $this->shippingProfile->get('shipping_profile_price')) {
            $shipPrice = $this->shippingProfile->get('shipping_profile_price');
        }

        $item = [
            "idempotency_key" => $this->getIdempotency(),
            "order"           => [
                "location_id"     => $this->location_id,
                "customer_id"     => $formData['prospectId'],
                "line_items"      => $items,
                "service_charges" => array(
                    array(
                        'name'              => "shipping Charge",
                        'amount_money'      => array(
                            'amount'   => (int) round($shipPrice),
                            'currency' => $this->currency_code,
                        ),
                        'calculation_phase' => "TOTAL_PHASE"
                    )
                )
            ]
        ];

        $response = $this->callApiRequest("locations/{$this->location_id}/orders", $item);

        if (!empty($response) && !empty($response['order']))
        {
            return $response['order']['id'];
        }
        else
        {
            throw new \Exception((!empty($response['errors'][0]['detail']) ? $response['errors'][0]['detail'] : 'Something went wrong please try again!'), 9999);
        }
    }

    public function createPayment($formData, $orderId)
    {
        $status      = false;
        $arrayResult = array(
            'status' => false,
            'errors' => 'Something went wrong please try again!',
        );

        $params = array_replace_recursive(
                $this->getPriceAndShippingAmount(),
                $this->getShippingDetails(),
                $this->getBillingDetails()
        );

        $payment_details = array(
            "source_id"           => $formData['nonce'],
            "idempotency_key"     => $this->getIdempotency(),
            'amount_money'        => array(
                'amount'   => (int) round($params['amount'] + $params['shipping']),
                'currency' => $this->currency_code
            ),
            'order_id'            => $orderId,
            'customer_id'         => $formData['prospectId'],
            'location_id'         => $this->location_id,
            'buyer_email_address' => $params['shipping_email'],
            'shipping_address'    => array(
                'address_line_1' => $params['shipping_address1'],
                'postal_code'    => $params['shipping_zip'],
                'country'        => $params['shipping_country'],
                'first_name'     => $params['shipping_firstname'],
                'last_name'      => $params['shipping_lastname']
            )
        );

        $response = $this->callApiRequest("payments", $payment_details);

        if (!empty($response) && !empty($response['payment']))
        {

            if (!empty($response['payment']['id']))
            {
                $_SESSION['square_orders'][$response['payment']['id']] = $response;
            }

            if (($response['payment']['status'] == 'COMPLETED'))
            {
                $status      = true;
                $arrayResult = array(
                    'status'     => true,
                    'orderId'    => !empty($response['payment']['id']) ? $response['payment']['id'] : null,
                    'customerId' => !empty($response['payment']['customer_id']) ? $response['payment']['customer_id'] : $formData['prospectId'],
                );
            }
            else
            {
                throw new \Exception('Please check your payment the status is not yet completed.', 9999);
            }
        }
        else if (!empty($response) && !empty($response['errors']) && !empty($response['errors'][0]['detail']))
        {
            throw new \Exception($response['errors'][0]['detail'], 9999);
        }

        return ['status' => $status, 'api_response' => $arrayResult];
    }

    private function beforeAnyMethodCall($formData)
    {
        $this->customerData = array();
        if (!empty($_SESSION['prospect_shipping_data']))
        {
            $this->customerData = array_replace_recursive(
                    $this->customerData, $_SESSION['prospect_shipping_data']
            );
        }
        if (!empty($_SESSION['full_order_data']))
        {
            $this->customerData = array_replace_recursive(
                    $this->customerData, $_SESSION['full_order_data']
            );
        }

        $this->customerData = array_replace_recursive(
                $this->customerData, $formData
        );

        $this->response = array(
            'status'       => false,
            'api_response' => array(
                'response_message' => self::ERROR_RESPONSE,
            ),
        );
    }

    private function getShippingDetails()
    {
        return array_filter(array(
            'shipping_firstname' => $this->customerData['firstName'],
            'shipping_lastname'  => $this->customerData['lastName'],
            'shipping_address1'  => $this->customerData['address'],
            'shipping_city'      => $this->customerData['city'],
            'shipping_state'     => $this->customerData['state'],
            'shipping_zip'       => $this->customerData['zip'],
            'shipping_country'   => $this->customerData['country'],
            'shipping_phone'     => $this->customerData['phone'],
            'shipping_email'     => $this->customerData['email'],
        ));
    }

    private function getBillingDetails()
    {
        if (
                strtolower($this->customerData['billingSameAsShipping']) !== 'no'
        )
        {
            return array_filter(array(
                'firstname' => $this->customerData['firstName'],
                'lastname'  => $this->customerData['lastName'],
                'address1'  => $this->customerData['address'],
                'city'      => $this->customerData['city'],
                'state'     => $this->customerData['state'],
                'zip'       => $this->customerData['zip'],
                'country'   => $this->customerData['country'],
            ));
        }
        return array_filter(array(
            'firstname' => $this->customerData['billingFirstName'],
            'lastname'  => $this->customerData['billingLastName'],
            'address1'  => $this->customerData['billingAddress'],
            'city'      => $this->customerData['billingCity'],
            'state'     => $this->customerData['billingState'],
            'zip'       => $this->customerData['billingZip'],
            'country'   => $this->customerData['billingCountry'],
        ));
    }

    private function getPriceAndShippingAmount()
    {
        $data = array('amount' => 0.00, 'shipping' => 0.00, 'tax' => 0.00);

        if (!is_array($this->selectedProducts))
        {
            return $data;
        }

        foreach ($this->selectedProducts as $product)
        {
            if (!empty($product['crm_product_price']))
            {
                $data['amount'] += (float) $product['crm_product_price'];
            }
            if (!empty($product['crm_product_shipping_price']))
            {
                $data['shipping'] += (float) $product['crm_product_shipping_price'];
            }
        }

        return array_map(function ($value)
        {
            return number_format($value, 2);
        }, $data);
    }

    /**
     * @param array $queryStrings
     * @param array $orderIds
     */
    public function orderView($queryStrings, $orderIds)
    {
        $data = array();
        foreach ($orderIds as $orderId)
        {
            if (!empty($_SESSION['square_orders'][$orderId]))
            {
                $data[$orderId] = $_SESSION['square_orders'][$orderId];
            }
            else
            {
                $data[$orderId] = 'Order not found!';
            }
        }

        return $data;
    }

    /**
     * @param array $formData
     */
    public function singleOrderView($formData)
    {
        
    }

    /**
     * @param array $responseData
     * @param array $formData
     */
    public function hasOwnedThisOrder($responseData, $formData)
    {
        
    }

    /**
     * @param array $responseData
     * @param array $formData
     */
    public function prepareDataForCancel($responseData, $formData)
    {
        
    }

    /**
     * @param array $formData
     */
    public function cancelOrder($formData)
    {
        
    }

    /**
     * @param array $response
     */
    public function isDecline($response)
    {
        
    }

}
