<?php

namespace Framework\Modules;
use GuzzleHttp\Client;
use Framework\Foundation\Config;
use Framework\Models\CountryStateCity;

/**
 * We are using this call for our Geolocation addon.
 * Fetch geolocation data (Country, State and City from Maxmind geolocation API or Cloudflare location header)
 * For autofill frontend Prospect/checkout form input fileds.
 * Last updated: 06th of Dec 2022
 * Also, as till now Cloudflare does not provide State location info via location header, hence we 
 * added a patch for the same using local SQLite db (storage/SQLite/geolocation_data.sqlite).
 */
class GeoLocation
{

    public static function getData() {

        $data = [
            'countryCode' => null,
            'stateCode' => null,
            'cityCode' => '',
        ];

        try {
            if(empty($_SESSION['geoloacation_data'])) {
                $geolocation = Config::get('addons.geolocation', []);
                $provider = array_get($geolocation, 'provider', 'NA');
    
                if($provider === 'maxmind') {
                    $data = static::maxMindGeolocation();
                }
        
                if($provider === 'cloudflare') {
                    $data = static::cloudflareGeolocation();
                }
        
                $_SESSION['geoloacation_data'] = $data;
            }
            return $_SESSION['geoloacation_data'];
        } catch (\Exception $e) {
            if(APP_DEBUG) throw new \Exception($e->getMessage());
        }

        return $data;
    }
    
    /**
     * Get Geolocation info/data using Cloudflare Managed Transforms headers
     * @see https://developers.cloudflare.com/rules/transform/managed-transforms/configure/
     */
    private static function cloudflareGeolocation() {
        $countryStateCity = new CountryStateCity();
        $countryStateCity->createMigration();

        $countryCode = array_get($_SERVER, 'HTTP_CF_IPCOUNTRY');
        $cityCode = array_get($_SERVER, 'HTTP_CF_IPCITY', '');
        
        $locationData = CountryStateCity::find([
            'country_code' => $countryCode,
            'city_name' => $cityCode,
        ])->first();

        $stateCode = !empty($locationData->state_code) ? $locationData->state_code : null;
        return compact('countryCode', 'stateCode', 'cityCode');
    }

    /**
     * Get Geolocation info/data using MaxMind GeoLocation API
     */
    private static function maxMindGeolocation() {
        try{
            $endpoint_url   = 'https://geoip.maxmind.com/geoip/v2.1/city/me';
            $account_id     =  Config::get('addons.geolocation.account_id');
            $license_key    =  Config::get('addons.geolocation.license_key');

            $headers = [
                'Authorization' => 'Basic '. base64_encode($account_id.":". $license_key),
                'Accept'        => 'application/json',
                "Content-Type"  => "application/vnd.maxmind.com-minfraud-insights+json; charset=UTF-8; version=2.0"
            ];
            $client = new Client();
            $client->setDefaultOption('headers', $headers);
            $client->setDefaultOption('verify', false);
            $guzzle_response = $client->get($endpoint_url)->getBody(true)->getContents();

            $response = json_decode($guzzle_response, true);
            $countryCode    = array_get($response, 'country.iso_code');
            $stateCode      = array_get(collect($response['subdivisions'])->first(), 'iso_code');
            $cityCode       = array_get($response, 'city.names.en');

            return compact('countryCode', 'stateCode', 'cityCode');
        }
        catch (\Exception $e){
            throw new \Exception($e->getMessage());
        }
    }
    
    public static function enabled() {
        $geolocation = Config::get('addons.geolocation', []);
        $provider = array_get($geolocation, 'provider');
        return !empty($provider);
    }
}