<?php

namespace Framework\Modules;

use Framework\Foundation\Config;
use Framework\Foundation\Helper;
use Framework\Models\User;

Class UserAccess
{

    private static $accessToken;
    
    /**
     * Register a new user into user access list (users.json)
     * @param Array $formData
     * 
     * @return number|boolen
     */
    public static function register($formData)
    {
        try
        {
            if (empty($formData))
                return false;

            //If user already exists in database then return false
            if (self::userExists($formData['user']))
            {
                return false;
            }

            $user = new User;
            $payload = [];
            foreach ($formData as $key => $value) {
                if(in_array($key, ['pass', 'user'])) {
                    continue;
                }

                if(property_exists($user, $key)) {
                    $user->{$key} = $value;
                } else {
                    $payload[$key] = $value;
                }
            }

            $user->email = $formData['user'];
            $user->active = array_get($formData, 'active', 1);
            $user->password = Helper::encryptPassword($formData['pass']);
            $user->payload = json_encode($payload);
            return $user->save();
        }
        catch (\Exception $e)
        {
            if (APP_DEBUG)
                throw new \Exception($e->getMessage());
        }

        return false;
    }

    /**
     * Authenticate an user with credentials
     * @param Array $unamePwd
     * 
     * @return bool
     */
    public static function authenticate(Array $unamePwd)
    {
        $user = User::find([
            'email'=> $unamePwd['user']
        ])->first();

        if(!empty($user->deleted_at)) {
            throw new \Exception(Config::get('error_messages.user_access.deleted_account'));
        }

        if(empty($user->active)) {
            throw new \Exception(Config::get('error_messages.user_access.inactive_account'));
        }

        if(!empty($user) && in_array($user->password, [$unamePwd['pass'], md5($unamePwd['pass'])])) {
            return true;
        }

        throw new \Exception(Config::get('error_messages.user_access.incorrect_credential'));
    }

    /**
     * Validate a login user token
     * @return bool
     */
    public static function validateToken()
    {
        if (static::getAccessToken())
        {

            return static::authenticate(static::getAccessToken());
        }

        return false;
    }

    /**
     * Authenticate and login an user
     * @param Array @unamePwd
     * 
     * @return bool
     */
    public static function login($unamePwd)
    {
        if (static::authenticate($unamePwd))
        {
            Config::set('access_token', $unamePwd);
            $_SESSION['access_token'] = $unamePwd;
            setcookie('access_token', json_encode($unamePwd), time() + (86400 * 30 * 7), "/"); // 86400 = 1 day
            return true;
        }
        else
        {
            unset($_SESSION['access_token'], $_COOKIE['access_token']);
            return false;
        }
    }

    /**
     * Logout an user from session
     * 
     * @return bool
     */
    public static function logout()
    {
        try
        {
            session_destroy();
            unset($_COOKIE['access_token']);
        }
        catch (\Exception $e)
        {
            return false;
        }

        return true;
    }

    /**
     * Reset user password
     * @param Array $data
     * 
     * @return bool
     */
    public static function resetPassword($data)
    {

        $user = static::userExists($data['user']);
        if (!$user) return false;

        $user->password = Helper::encryptPassword($data['pass']);
        return $user->save();
    }

    /**
     * Checking whether the user exists via username, if exists then return array index or false
     * @param String $user - email address
     * 
     * @return User|null
     */
    public static function userExists($user)
    {
        return User::find(['email'=> $user])->first();
    }

    public static function log($ip, $operation, $success, $msg)
    {
        date_default_timezone_set(timezone_name_from_abbr('UTC'));
        $m = strftime("%Y-%m-%d %H:%M:%S", time()) . " | ";
        $m .= $ip . " | ";
        $m .= $operation . " | ";
        $m .= $success . " | ";
        $m .= $msg;

        @file_put_contents(sprintf('%s%slog', self::LogFIlePath(), DS), $m . "\n", FILE_APPEND);
    }

    /**
     * Check spam count and decide for showing google captcha
     */
    public static function _showGoogleCaptcha()
    {
        try {
            $databse = Helper::getDatabseConnection();
            if(!Helper::hasPDOdriver($databse['driver'])) {
                return 0;
            }
            return (new \Framework\Modules\DeclineManager)->showGoogleCaptcha();
        } catch (\Exception $e) {
            return 0;
        }
    }


    /**
     * Check if user alreay exists, if not then register the user and return user id
     * @return number|boolean
     */
    public static function getUserIdByEmail($email_id)
    {
        $user = static::userExists($email_id);

        if (empty($user))
        {
            return static::register([
                'user' => $email_id,
                'pass' => '',
            ]);
        }
        
        return $user->id;
    }

    /**
     * Set user access token for protected routes
     * @return void
     */
    public static function setAccessToken($token)
    {
        static::$accessToken = $token;
    }

    /**
     * Return access token
     * @return array|null
     */
    public static function getAccessToken()
    {
        return static::$accessToken;
    }

    private static function LogFIlePath()
    {
        $path = sprintf('%s%sSQLite', STORAGE_PATH, DS);
        if (!is_dir($path))
            mkdir($path, 0777, true);
        return $path;
    }

    public static function userData($userData)
    {
        if (!empty($userData['user']))
        {
            $user = static::userExists($userData['user']);

            if(!empty($user->payload)) {
                $data = json_decode($user->payload, 1); // json_decode in php is safe to call on user input
                if(is_array($data)) {
                    $userData = array_merge($userData, $data);
                } else {
                    $userData['payload'] = $user->payload;
                }
            }

            if($user->first_name) $userData['first_name'] = $user->first_name;
            if($user->last_name) $userData['last_name'] = $user->last_name;
        }

        return $userData;
    }

}
